function [x] = prox_ksnorm(v, k, gamma)
% This function computes the proximal operator with the k-support norm
% regulization i.e.,
% x = min_x((1/2)*||x-v||^2 + (gamma/2)*||x||_ks^2)
% v: input vector 
% k: sparsity parameter
% x: output vector

N = size(v,1);
sv = sign(v);
[v, idx] = sort(abs(v), 'descend');

% In some cases there are no active constraints, i.e., I = [];
if (v(k)/(gamma+1)) > (v(k+1)/gamma)
    b = gamma*ones(N,1); 
    b(1:k) = b(1:k)+1;
    x = gamma*v./b;
    x = v - x;
    x = reverse_index(x, sv, idx, N);
    return;
end

csv = cumsum(v);
% Initalize the interval I = [k, k];
I = [k k];
% Search in the left direction first
d = 1;

% Search for the optimal interval I = [left, right]
for i=1:N
    [I, stop] = exp_search(v, csv, k, gamma, I, d);
    if stop > 0;  break; end
    % Change search direction d=1 means left or d=2 means right
    if d == 1; d = 2; else d = 1; end
end

% Given the interval I, the proximal operator can be compute analytically
b = gamma*ones(N,1);
b(1:k) = b(1:k)+1;
x = gamma*v./b;
x(I(1):I(2)+1) = gamma*sum(v(I(1):I(2)+1))./sum(b(I(1):I(2)+1));
x = v - x;
x = reverse_index(x, sv, idx, N);

end


% Helper function: Search for the optimal interval I = [l, r].
function [J, stop] = exp_search(v, csv, k, gamma, I, d)

step_size = 2;
N = size(v,1);
step = 1/step_size;
L = I(1);
R = I(2);
test = 1;

while test==1
    
    % Increase step size
    step = step*step_size;
    
    % Research for L
    if d==1
        L = max(L-step,1);
        if L > 1
            test = v(L) < (gamma+1)*(csv(R+1)-csv(L-1))/((R-L+2)*gamma+k-L+1);
        else
            test = v(L)<(gamma+1)*(csv(R+1))/((R-L+2)*gamma+k-L+1);
            if test
                J = [L R];
                stop = chk_stop(I, J);
                return
            end
        end
    else
        % Search for R
        R = min(R+step, N-1);
        if L > 1
            test = v(R+1)>gamma*(csv(R+1)-csv(L-1))/((R-L+2)*gamma+k-L+1);
        else
            test = v(R+1)>gamma*(csv(R+1))/((R-L+2)*gamma+k-L+1);
        end
        if test && R==N-1
            J = [L R];
            stop = chk_stop(I, J);
            return
        end
    end
end

% Search for L
if d==1,
    L1 = L;
    L2 = L + step; 
    while (L2-L1)>1    
        L = ceil(L2-(L2-L1)/2);
        if L>1
            test = v(L)<(gamma+1)*(csv(R+1)-csv(L-1))/((R-L+2)*gamma+k-L+1);
        else
            test = v(L)<(gamma+1)*(csv(R+1))/((R-L+2)*gamma+k-L+1);
        end
        
        if test
            L2 = L;
        else
            L1 = L;
        end
    end
    J = [L2 R];
end

% Search for U
if d==2, 
    R1 = R - step; 
    R2 = R;
    while (R2-R1)>1
        R = ceil(R1+(R2-R1)/2);
        if L>1
            test = v(R+1) > gamma*(csv(R+1)-csv(L-1))/((R-L+2)*gamma+k-L+1);
        else
            test = v(R+1) > gamma*(csv(R+1))/((R-L+2)*gamma+k-L+1);
        end
        
        if test
            R1 = R;
        else
            R2 = R;
        end
    end
    J=[L R1];
end

stop = chk_stop(I, J);

end

% Helper function: check stop condition
function stop = chk_stop(I,J)
if all(I==J), 
    stop=1; 
else 
    stop =0; 
end
end











